package com.famoco.barcodeexternaltest;


import android.content.Context;
import android.os.Build;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.View;
import android.widget.EditText;

import com.zebra.adc.decoder.BarCodeReader;

import static com.zebra.adc.decoder.BarCodeReader.ParamNum.LASER_ON_PRIM;

/**
 * This app tests the barcode scanner on FX300.
 * It relies on the image containing the shared libraries in:
 * system/lib/
 */
public class BarcodeExternalTestActivity extends AppCompatActivity implements
        BarCodeReader.DecodeCallback, BarCodeReader.ErrorCallback, View.OnClickListener {

    private static final String TAG = "BarcodeSdkDemo";

    private static final int PARAM_NUM1 = 765;
    private static final int PARAM_VAL1 = 0;
    private static final int PARAM_NUM_TIMEOUT = LASER_ON_PRIM; // num 136, scan timeout
    private static final int PARAM_VAL_TIMEOUT = 990; // unit 0.1 seconds, range 0.5 to 9.9 seconds

    static {
        try {
            System.loadLibrary("IAL");
            System.loadLibrary("SDL");
            System.loadLibrary("barcodereader44"); // Android 4.4
            Log.i(TAG, "Barcode scanner drivers loaded");
        } catch (Exception e) {
            Log.e(TAG, "static initialization  block", e);
        }
    }

    enum Mode{
        UNAVAILABLE,
        IDLE,
        SCANNING,
    }

    private Mode mode = null;
    private BarCodeReader bcr = null;
    private Context context = null;

    // life cycle //

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        Log.i(TAG, "onCreate");

        Log.d("MODEL",""+ Build.MODEL);
        if(this.getPackageManager().hasSystemFeature("android.hardware.barcode")){
            Log.d("FEATURE","BARCODE");
        }else{
            Log.d("FEATURE","NO_BARCODE");
        }

        setContentView(R.layout.activity_barcode_external_test);
        findViewById(R.id.button).setOnClickListener(this);
        mode = Mode.UNAVAILABLE;
        context = this;
    }

    @Override
    protected void onStart() {
        super.onStart();
        Log.i(TAG, "onStart");
        initBarcode();
    }

    @Override
    protected void onStop() {
        super.onStop();
        Log.i(TAG, "onStop");
        releaseBarcode();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        Log.i(TAG, "onDestroy");
    }

    // call-backs //

    @Override
    public void onClick(View arg0) {
        Log.i(TAG, "onClick");
        switch (mode) {
            case IDLE: startScan(); clearText(); break;
            case SCANNING: stopScan(); break;
            case UNAVAILABLE: /* do nothing */ break;
        }
    }


    @Override
    public void onDecodeComplete(int symbology, int length, byte[] data, BarCodeReader reader) {
        Log.i(TAG, "onDecodeComplete");
        if (length == 0 || data == null || data.length == 0) {
            Log.i(TAG, "ignore");
            return;
        }
        try {
            Log.i(TAG, "decode");
            String decodedText = new String(data).substring(0, length);
            handleScanResult(decodedText);
        } catch (Exception e) {
            Log.e(TAG, "onDecodeComplete", e);
        }
    }

    @Override
    public void onError(int error, BarCodeReader reader) {
        Log.i(TAG, "onError");
    }


    @Override
    public void onEvent(int event, int info, byte[] data, BarCodeReader reader) {
        Log.i(TAG, "onEvent");
    }


    // barcode //

    /**
     * 1. Initialize the barcode scanner
     */
    private void initBarcode() {
        Log.i(TAG, "initBarcode");
        bcr = BarCodeReader.open(1, context); // Android 4.3 and above
        if (bcr == null) throw new RuntimeException("cant open barcode");
        bcr.setDecodeCallback(this);
        bcr.setErrorCallback(this);
        /* this parameters are given by James from the HW provider.
         the setParameter method reports an error while trying to set them... */
        bcr.setParameter(PARAM_NUM1, PARAM_VAL1);
        bcr.setParameter(PARAM_NUM_TIMEOUT, PARAM_VAL_TIMEOUT);
        mode = Mode.IDLE;
    }

    /**
     * 2. Start a barcode scan
     */
    private void startScan() {
        Log.i(TAG, "startScan");
        bcr.startDecode();
        mode = Mode.SCANNING;
    }

    /**
     * 3. Stop a barcode scan
     */
    private void stopScan() {
        Log.i(TAG, "stopDecode");
        bcr.stopDecode();
        mode = Mode.IDLE;
    }

    /**
     * 4. This function handle the decoded scan result.
     */
    private void handleScanResult(String decodedText) {
        ((EditText)findViewById(R.id.text)).setText(decodedText);
        mode = Mode.IDLE;
    }

    /**
     * 5. Release the barcode scanner.
     */
    private void releaseBarcode () {
        Log.i(TAG, "releaseBarcode");
        bcr.setDecodeCallback(null);
        bcr.setErrorCallback(null);
        bcr.release();
        bcr = null;
        mode = Mode.UNAVAILABLE;
    }

    // helpers //

    private void clearText() {
        Log.i(TAG, "clearText");
        ((EditText)findViewById(R.id.text)).setText("");
    }

}
